
# Libraries & Working Directory ----
library(tidyverse)
library(readxl) #To load Excel files
library(pROC) #To produce ROC curves & calculate AUC
library(binom) #To produce Wilson confidence intervals
library(gt) #To produce tables
library(lpSolve) #To load library(irr)
library(irr) #To produce Kappa

# Please set path as appropriate to the same folder where data files are contained
setwd("~/Library/CloudStorage/Dropbox/Work/Alexander Witkowski/Data")

## Load data ----
org_data <- read_xlsx("dermatologists.xlsx") #Load dataset

## Restructure data -----
colnames(org_data) <- org_data[2,] #swap column names with second row values

data <- as.data.frame(org_data[-c(1,3,6,7,9),-1]) #remove unnecessary rows & columns

rownames(data)[1] <- "case" #plenty of renaming
rownames(data)[2] <- "P0"
rownames(data)[3] <- "Px"
rownames(data)[4] <- "Qx"
rownames(data)[5:44] <- paste0(seq(1:40), "_", data[5:44,1])
data <- data[,-1] #remove unnecessary column

data <- as.data.frame(t(data)) #transpose dataset
data$study <- rownames(data) #separate a column to contain study ID 
rownames(data) <- seq_len(nrow(data)) #rename rows with sequence of numbers

data$study[1:200] <- "E1" #assign study names | 4 questions here, 50 cases
data$study[201:450] <- "E2" #Q1-5 here, 50 cases
data$study[451:700] <- "E3" #Q1-5 here, 50 cases
data$study[701:800] <- "E4" #Q1-5 here too, but only 20 cases

table(data$study) #Assess study assignment

data <- data %>% 
  select(study, everything()) %>% #place study column as first
  mutate(Qx = recode(Qx,          #swap long questions with Q1-Q5
                     "DIAGNOSIS" = "Q1",
                     "CONF DIAGNOSIS" = "Q2",
                     "MANAGEMENT" = "Q3",
                     "CONF MANAGEMENT" = "Q4",
                     "D3PC MOST PREDOMINANT" = "Q5"))

datalong <- data %>%              #Convert to long form data
  pivot_longer(cols = 6:45,
               names_to = "Observer",
               values_to = "Assessment",
               values_drop_na = FALSE)

datalong <- datalong %>%          #Separate Observer ID & Profession
  separate(col = Observer,
           into = c("Obs_ID", "Obs_Prof"),
           sep = "_")

datalong$case <- as.double(datalong$case) #change data type
datalong$Obs_ID <- as.double(datalong$Obs_ID)

datalong <- datalong %>% #arrange dataset
  arrange(study, case, Obs_ID, Qx)

datalong <- datalong %>% #create a unique ID that neglects the Qx
  mutate(UID = paste0(study, "_", case, "_", Obs_ID))

datalong_1 <- filter(datalong, Qx == "Q1") #separate Qx datasets by value
datalong_2 <- filter(datalong, Qx == "Q2")
datalong_3 <- filter(datalong, Qx == "Q3")
datalong_4 <- filter(datalong, Qx == "Q4")
datalong_5 <- filter(datalong, Qx == "Q5")

### FIRST FINDATA -----
findata <- datalong_1 %>%                  #merge Qx datasets
  left_join(datalong_2 %>% select(UID, Assessment), by = "UID", suffix = c("_1", "_2"), multiple = "first") %>% 
  left_join(datalong_3 %>% select(UID, Assessment), by = "UID", suffix = c("_2", "_3"), multiple = "first") %>%
  left_join(datalong_4 %>% select(UID, Assessment), by = "UID", suffix = c("_3", "_4"), multiple = "first") %>%
  left_join(datalong_5 %>% select(UID, Assessment), by = "UID", suffix = c("_4", "_5"), multiple = "first")

findata <- findata %>%                     #Reorder columns, remove UID & Qx
  select(-Qx, -UID) %>% 
  rename(Q1 = Assessment_1,
         Q2 = Assessment_2,
         Q3 = Assessment_3,
         Q4 = Assessment_4,
         Q5 = Assessment)                  #Now we have the final dataset

### Assign factor values ----
findata$P0 <- factor(findata$P0, levels = c(0, 1), labels = c("Benign", "Malignant"))
findata$Q1 <- factor(findata$Q1, levels = c("Benign", "Malignant"))
findata$Q2 <- factor(findata$Q2, levels = c("None", "Slight", "Moderate", "High"))
findata$Q3 <- factor(findata$Q3, levels = c("Monitor", "Referral", "Biopsy"))
findata$Q4 <- factor(findata$Q4, levels = c("None", "Slight", "Moderate", "High"))
findata$Q5 <- factor(findata$Q5, levels = c("Not applicable",
                                            "Asymmetry and/or atypical network",
                                            "Blue-white-grey-violet color(s)",
                                            "Round structures"))
findata$study <- factor(findata$study, levels = c("E1", "E2", "E3", "E4"))

colSums(is.na(findata)) #Check N missing values across columns


# Removing residents from observers (they are added to primary care dataset) -----
findata <- findata %>%  
  subset(Obs_Prof != "Resident")
table(findata$Obs_Prof)
### Adding AI-Benign cases from E3 to E4 ----
finE3Ben <- findata %>%  # Separating benign cases
  filter(study == "E3" & Q1 == "Benign") %>% 
  mutate(study = ifelse(study == "E3", "E4", study))
findata <- rbind(findata, finE3Ben) # Binding benign cases to heatmaps dataset (to assess specificity)

## Logistic Regressions

### Diagnostic Prediction Binary Logistic #E4 censored ----

# Naked eye exam
model1_dx <- glm(data = filter(findata, study == "E1"), 
                 formula = P0 ~ Q1 + Q2,
                 family =  binomial)

OR_tab_dx1 <- cbind(exp(coef(model1_dx)), exp(confint(model1_dx))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID")

# Dermoscopy image
model2_dx <- glm(data = filter(findata, study == "E2" & !is.na(Q1) & !is.na(Q2)),
                 formula = P0 ~ Q1 + Q2,
                 family =  binomial)

OR_tab_dx2 <- cbind(exp(coef(model2_dx)), exp(confint(model2_dx))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID")

# Dermoscopy image + AI interpretation
model3_dx <- glm(data = filter(findata, study == "E3" & !is.na(Q1) & !is.na(Q2)),
                 formula = P0 ~ Q1 + Q2,
                 family =  binomial)

OR_tab_dx3 <- cbind(exp(coef(model3_dx)), exp(confint(model3_dx))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID")


OR_dxtab <- OR_tab_dx1 %>%
  full_join(OR_tab_dx2, by = "ID") %>%
  full_join(OR_tab_dx3, by = "ID") %>%
  mutate(across(where(is.numeric), ~ round(., 2)))

gt(OR_dxtab)
# ROC/AUC ----
par(pty = "s") #set graphical parameter to show 'square' plots (s = square)

dx_roc1 <- roc(filter(findata, study == "E1")$P0,
               model1_dx$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 40,
               col = "#e60049",
               lwd = 4,
               identity.lty= 3,
               identity.lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE)

dx_roc2 <- roc(filter(findata, study == "E2" & !is.na(Q1) & !is.na(Q2))$P0,
               model2_dx$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 35,
               col = "#0d88e6",
               lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE,
               add = TRUE)

dx_roc3 <- roc(filter(findata, study == "E3" & !is.na(Q1) & !is.na(Q2))$P0,
               model3_dx$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 30,
               col = "#00bfa0",
               lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE,
               add = TRUE)

legend(x = 80, y = 15, legend = c("Clinical Image Alone",
                                 "Clinical & Dermoscopy Images",
                                 "Clinical & Dermoscopy Images + AI Interpretation"),
       col = c("#e60049", "#0d88e6", "#00bfa0"), lwd = 4, cex =0.6)

# AUC 95% CI
ci.auc(dx_roc1)
ci.auc(dx_roc2)
ci.auc(dx_roc3)

# ROC.TEST ----
roc.test(dx_roc1, dx_roc2, method = "bootstrap", paired = TRUE) #Compare E1 & E2 ROCs
roc.test(dx_roc2, dx_roc3, method = "bootstrap", paired = TRUE) #Compare E2 & E3 ROCs

### Decision Prediction Binary Logistic MODELS #E4 censored ----
mandata <- findata %>% #Swap referral, biopsy & monitor with sus & nonsus
  mutate(Q3 = case_when(Q3 %in% c("Referral", "Biopsy")~ "sus",
                        Q3 == "Monitor" ~ "nosus"))

model1_dcs <- glm(data = filter(mandata, study == "E1"),
                  P0 ~ Q3 + Q4,
                  family =  binomial)

model2_dcs <- glm(data = filter(mandata, study == "E2" & !is.na(Q3)),
                  P0 ~ Q3 + Q4,
                  family =  binomial)

# For Decision E3, REASSIGN confidence levels
mandata$Q4 <- factor(mandata$Q4, levels = c("Slight", "Moderate", "High"))

model3_dcs <- glm(data = filter(mandata, study == "E3" & !is.na(Q4)),
                  P0 ~ Q3 + Q4,
                  family =  binomial)
summary(model3_dcs)

# Model summary ----
summary(model1_dcs)
summary(model2_dcs)
summary(model3_dcs)

# Build aOR table -----
OR_tab_dcs1 <- cbind(exp(coef(model1_dcs)), exp(confint(model1_dcs))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID")

OR_tab_dcs2 <- cbind(exp(coef(model2_dcs)), exp(confint(model2_dcs))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID")

OR_tab_dcs3 <- cbind(exp(coef(model3_dcs)), exp(confint(model3_dcs))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID")

OR_dcstab <- OR_tab_dcs1 %>%
  full_join(OR_tab_dcs2, by = "ID") %>%
  full_join(OR_tab_dcs3, by = "ID") %>%
  mutate(across(where(is.numeric), ~ round(., 2)))

gt(OR_dcstab)
# ROC/AUC ----
par(pty = "s") #set graphical parameter to show 'square' plots (s = square)

dcs_roc1 <- roc(filter(mandata, study == "E1")$P0,
               model1_dcs$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 40,
               col = "#e60049",
               lwd = 4,
               identity.lty= 3,
               identity.lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE)

dcs_roc2 <- roc(filter(mandata, study == "E2" & !is.na(Q1) & !is.na(Q2))$P0,
               model2_dcs$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 35,
               col = "#0d88e6",
               lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE,
               add = TRUE)

dcs_roc3 <- roc(filter(mandata, study == "E3" & !is.na(Q4))$P0,
               model3_dcs$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 30,
               col = "#00bfa0",
               lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE,
               add = TRUE)

# AUC 95% CI
ci.auc(dcs_roc1)
ci.auc(dcs_roc2)
ci.auc(dcs_roc3)

legend(x = 80, y = 15, legend = c("Clinical Image Alone",
                                  "Clinical & Dermoscopy Images",
                                  "Clinical & Dermoscopy Images + AI Interpretation"),
       col = c("#e60049", "#0d88e6", "#00bfa0"), lwd = 4, cex =0.6)

# ROC.TEST ----
roc.test(dcs_roc1, dcs_roc2, method = "bootstrap", paired = TRUE) #Compare E1 & E2 ROCs
roc.test(dcs_roc2, dcs_roc3, method = "bootstrap") #Compare E2 & E3 ROCs

### Inter-rater Reliability of Confidence (Cohen's Kappa) ----
kappa_E1_2 <- kappa2(cbind(filter(findata, study == "E1")$Q2,
                           filter(findata, study == "E2")$Q2))
kappa_E1_2

kappa_E2_3 <- kappa2(cbind(filter(findata, study == "E2")$Q2,
                      filter(findata, study == "E3")$Q2))
kappa_E2_3

table(filter(refin, study == "E2")$Q4, #The confidence matrix table
      filter(refin, study == "E3")$Q4)

### Sensitivity, Specificity, Wilson CI ----

refin <- findata %>%    # Code TP, FP, TN, FN
  mutate(Q1 = case_when(P0 == "Malignant" & Q1 == "Malignant" ~ "TP",
                        P0 == "Benign" & Q1 == "Malignant" ~ "FP",
                        P0 == "Malignant" & Q1 == "Benign" ~ "FN",
                        P0 == "Benign" & Q1 == "Benign" ~ "TN"),
         
         Q3 = case_when(P0 == "Malignant" & (Q3 == "Biopsy" | Q3 == "Referral") ~ "TP",
                        P0 == "Benign" & (Q3 == "Biopsy" | Q3 == "Referral") ~ "FP",
                        P0 == "Malignant" & Q3 == "Monitor" ~ "FN",
                        P0 == "Benign" & Q3 == "Monitor" ~ "TN"))

# building contingency tables ----
# Create 2x2 contingency tables for Diagnosis 
E1dx <- table(filter(refin, study == "E1")$Q1)
E2dx <- table(filter(refin, study == "E2")$Q1)
E3dx <- table(filter(refin, study == "E3")$Q1)

# Create 2x2 contingency tables for Decision 
E1dcs <- table(filter(refin, study == "E1")$Q3)
E2dcs <- table(filter(refin, study == "E2")$Q3)
E3dcs <- table(filter(refin, study == "E3")$Q3)

# Bind contingency tables
contab <- as.data.frame(rbind(E1dx, E2dx, E3dx,
                              E1dcs,E2dcs,E3dcs)) %>%
  mutate(Total = TP + FN + TN + FP,
         Malignant = TP + FN,
         Benign = TN + FP,
         Positive = TP + FP,
         Negative = TN + FN,
         Se = TP/(TP + FN),
         Sp = TN/(TN + FP))
SeCItab <- binom.wilson(contab$TP, contab$Malignant) %>% select(SeL = lower, SeU = upper)
SpCItab <- binom.wilson(contab$TN, contab$Benign) %>% select(SpL = lower, SpU = upper)

contab <- cbind(contab, SeCItab, SpCItab) %>%
  rownames_to_column(var = "Study") %>% 
  select(Study, TP, FN, FP, TN, Benign, Malignant, Positive, Negative, Total,
         Se, SeL, SeU, Sp, SpL, SpU) %>% 
  mutate(Se = round(Se*100,1), SeL = round(SeL*100,1), SeU = round(SeU*100,1),
         Sp = round(Sp*100,1), SpL = round(SpL*100,1), SpU = round(SpU*100,1))
gt(contab)

### Lesion demographics are published under the study with Dentists 🦷----
### Demographics ----
demodata <- read_xlsx("dermatologist demographics.xlsx")
table(demodata$Gender)
table(demodata$`Time practicing`)
table(demodata$`Type of practice`)
table(demodata$`Area of practice`)
table(demodata$`Biopsy frequency`)
table(demodata$Competence)
table(demodata$`Do you perform skin checks`)
table(demodata$`Derm referral frequency`)
table(filter(findata, study == "E1" & case == 1)$Obs_Prof)


### #Other Specific Stats -----
# 2x2 for Melanoma
Mel <- refin %>% 
  filter(study == "E3", Px %in% c("MM", "MIS"))

table(Mel$Px, Mel$Q3)
summary(table(Mel$Px, Mel$Q3))

# 2x2 for AMN
Nev <- refin %>% 
  filter(study == "E4", Px %in% c("AMN(M)", "AMN(S)", "AMN(SPITZ)"))

table(Nev$Px, Nev$Q3)
summary(table(Nev$Px, Nev$Q3))

table(filter(refin, study == "E3")$Q3)
summary(table(filter(refin, study == "E3")$Q3))

### Specific Features Only Models ----
# MODELS
qmodel2_dx <- glm(data = filter(findata, study == "E2" & !is.na(Q5)),
                 formula = P0 ~ Q5,
                 family =  binomial)

qmodel3_dx <- glm(data = filter(findata, study == "E3" & !is.na(Q5)),
                  formula = P0 ~ Q5,
                  family =  binomial)

qmodel4_dx <- glm(data = filter(findata, study == "E4" & !is.na(Q5)),
                 formula = P0 ~ Q5,
                 family =  binomial)

# TABLES
qtab_dx2 <- cbind(exp(coef(qmodel2_dx)), exp(confint(qmodel2_dx))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID") %>% 
  rename(var = ID,
         OR2 = V1,
         L2 = "2.5 %",
         U2 = "97.5 %")

qtab_dx3 <- cbind(exp(coef(qmodel3_dx)), exp(confint(qmodel3_dx))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID") %>% 
  rename(var3 = ID,
         OR3 = V1,
         L3 = "2.5 %",
         U3 = "97.5 %")

qtab_dx4 <- cbind(exp(coef(qmodel4_dx)), exp(confint(qmodel4_dx))) %>% 
  as.data.frame() %>% 
  rownames_to_column(var = "ID") %>% 
  rename(var4 = ID,
         OR4 = V1,
         L4 = "2.5 %",
         U4 = "97.5 %")

qtabs <- cbind(qtab_dx2, qtab_dx3, qtab_dx4) %>% 
  mutate(across(where(is.numeric), round, digits = 2))

gt(qtabs)

# AUC
### E2 v E3 v E4 Q5 AUC ----
par(pty = "s") #set graphical parameter to show 'square' plots (s = square)

q_roc2 <- roc(filter(findata, study == "E2" & !is.na(Q5))$P0,
               qmodel2_dx$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 40,
               col = "#e60049",
               lwd = 4,
               identity.lty= 3,
               identity.lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE)

q_roc3 <- roc(filter(findata, study == "E3" & !is.na(Q5))$P0,
               qmodel3_dx$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 35,
               col = "#0d88e6",
               lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE,
               add = TRUE)

q_roc4 <- roc(filter(findata, study == "E4" & !is.na(Q5))$P0,
               qmodel4_dx$fitted.values,
               legacy.axes = TRUE,
               xlim = c(100,0),
               percent = TRUE,
               print.auc = TRUE,
               print.auc.y = 30,
               col = "maroon",
               lwd = 4,
               print.thres = "all",
               print.thres.cex = 1.5,
               print.thres.pattern = "",
               plot = TRUE,
               add = TRUE)

legend(x = 80, y = 15, legend = c("MD3PC features seen using DDI",
                                  "MD3PC features seen using DDI-AI",
                                  "MD3PC features seen using DDI-AI Heatmaps"),
       col = c("#e60049", "#0d88e6", "maroon"), lwd = 4, cex =0.6)


# ROC 95% CI ----
ci.auc(q_roc2)
ci.auc(q_roc3)
ci.auc(q_roc4)

# ROC.TEST ----
roc.test(q_roc2, q_roc3, method = "bootstrap") #Compare E1 & E2 ROCs
roc.test(q_roc3, q_roc4, method = "bootstrap") #Compare E2 & E3 ROCs
roc.test(dx_roc3, dx_roc4, method = "bootstrap") #Compare E3 & E4 ROCs



